#!/usr/bin/python
# -*- coding: UTF-8 -*-
import os

import gensim
import numpy as np
from gensim.models.keyedvectors import Word2VecKeyedVectors

from sekg.ir.doc.wrapper import PreprocessMultiFieldDocumentCollection
from sekg.ir.models.base import DocumentSimModel
from sekg.model.word2vec.tune_word2vec import TunedWord2VecTrainer
from sekg.util.file import DirUtil


class AVGW2VFLModel(DocumentSimModel):
    __corpus_name__ = 'corpus.mm'
    __w2v_model_name__ = "word2vec.model"
    __avg_w2v_model_name__ = "word2vec.avg.model"
    __dictionary_name__ = 'dictionary.dict'
    __sim_index__ = "avgw2v.sim.index"

    __entity_collection_name__ = "doc.pre.collection"

    DEFAULT_EMBEDDING_SIZE = 100

    def __init__(self, name, model_dir_path, **config):

        """
        init the lsi model with
        :param model_dir_path:
        """
        super().__init__(name, model_dir_path, **config)
        self.w2v_model = None
        self.avg_w2v_model = None

        # self.avg_w2v_model_field_map = {}
        self.field_set = {}

        self.corpus = None
        self.dict = None
        self.similarity_index = None
        self.__init_sub_model_path__()
        self.preprocessor = None
        self.preprocess_doc_collection = None

        self.NP_VECTOR_NOT_EXIST = None
        self.embedding_size = 0
        self.__init_embedding_size(AVGW2VFLModel.DEFAULT_EMBEDDING_SIZE)

    def __init_sub_model_path__(self):
        if self.model_dir_path is None:
            return
        # init the paths of models
        self.corpus_path = os.path.join(self.model_dir_path, self.__corpus_name__)
        self.dictionary_path = os.path.join(self.model_dir_path, self.__dictionary_name__)
        self.w2v_model_path = os.path.join(self.model_dir_path, self.__w2v_model_name__)
        self.avg_w2v_model_path = os.path.join(self.model_dir_path, self.__avg_w2v_model_name__)

        # todo: fix the path name
        self.entity_collection_path = os.path.join(self.model_dir_path, self.__entity_collection_name__)

        self.sim_index_dir = os.path.join(self.model_dir_path, "index")

        self.sim_index_path = os.path.join(self.sim_index_dir, self.__sim_index__)

        DirUtil.create_file_dir(self.model_dir_path)
        DirUtil.create_file_dir(self.sim_index_dir)

    def __init_document_collection(self, preprocess_doc_collection):
        self.preprocess_doc_collection = preprocess_doc_collection
        self.preprocessor = preprocess_doc_collection.get_preprocessor()
        self.field_set = preprocess_doc_collection.get_field_set()

    def __init_embedding_size(self, embedding_size):
        self.embedding_size = embedding_size
        self.NP_VECTOR_NOT_EXIST = np.zeros(embedding_size)
        self.NP_VECTOR_NOT_EXIST[0] = 1e-07

    def init_model_as_submodel(self):
        print("init_sub_model_path")
        self.__init_sub_model_path__()

        print("loading doc_collection")
        preprocess_doc_collection = PreprocessMultiFieldDocumentCollection.load(self.entity_collection_path)
        self.preprocessor = preprocess_doc_collection.get_preprocessor()
        self.field_set = preprocess_doc_collection.get_field_set()

        print("loading the Word2vec models")
        self.w2v_model = gensim.models.keyedvectors.Word2VecKeyedVectors.load(self.w2v_model_path)

        self.avg_w2v_model = gensim.models.keyedvectors.Word2VecKeyedVectors.load(self.avg_w2v_model_path)

    def init_model(self):
        """
        init the model
        :return:
        """
        print("init_sub_model_path")
        self.__init_sub_model_path__()

        print("loading doc_collection")
        preprocess_doc_collection = PreprocessMultiFieldDocumentCollection.load(self.entity_collection_path)
        self.__init_document_collection(preprocess_doc_collection)

        print("loading the Word2vec models")
        self.w2v_model = gensim.models.keyedvectors.Word2VecKeyedVectors.load(self.w2v_model_path)

        self.avg_w2v_model = gensim.models.keyedvectors.Word2VecKeyedVectors.load(self.avg_w2v_model_path)

        # todo: load the index
        self.corpus = gensim.corpora.MmCorpus(self.corpus_path)
        self.dict = gensim.corpora.Dictionary.load(self.dictionary_path)
        print("All document number: ", self.dict.num_docs)
        print("All words number: ", self.dict.num_pos)

    def get_full_doc_score_vec(self, query):
        full_entity_score_vec = self.get_cache_score_vector(query)
        if full_entity_score_vec is not None:
            return full_entity_score_vec

        query_vec = self.string2vector(query)
        sim_score_vec = self.avg_w2v_model.similar_by_vector(query_vec, topn=None)
        full_entity_score_vec = (sim_score_vec + 1) / 2

        self.cache_entity_score_vector(query, full_entity_score_vec)

        return full_entity_score_vec

    def train_from_document_collection(self, preprocess_document_collection, embedding_size=100,
                                       pretrain_w2v_path=None, tune_pretrain_w2v=True, pretrain_binary=True):
        print("start training")
        self.__init_embedding_size(embedding_size)
        self.__init_document_collection(preprocess_document_collection)

        corpus_clean_text = []
        preprocess_multi_field_doc_list = preprocess_document_collection.get_all_preprocess_document_list()
        for docno, multi_field_doc in enumerate(preprocess_multi_field_doc_list):
            corpus_clean_text.append(multi_field_doc.get_document_text_words())

        print("corpus len=%d" % len(corpus_clean_text))

        print("Dictionary init...")
        self.dict = gensim.corpora.Dictionary(corpus_clean_text)
        print(" Dictionary init complete ")

        print("parse to bow corpus")
        self.corpus = [self.dict.doc2bow(text) for text in corpus_clean_text]
        print("parse to bow corpus complete")

        if pretrain_w2v_path is not None:
            print("pretrain Word2vec path is given, loading")
            if tune_pretrain_w2v == True:
                w2v_model = TunedWord2VecTrainer.tune(corpus_clean_text, pretrain_binary=pretrain_binary,
                                                      pretrain_w2v_path=pretrain_w2v_path)
                self.w2v_model = w2v_model.wv
                print("tune the pretrain w2v for model")
            else:
                if pretrain_binary == True:
                    pretrained_word2vec_model = Word2VecKeyedVectors.load_word2vec_format(
                        pretrain_w2v_path,
                        binary=pretrain_binary)
                    self.w2v_model = pretrained_word2vec_model

                else:
                    self.w2v_model = gensim.models.keyedvectors.Word2VecKeyedVectors.load(pretrain_w2v_path)

        else:
            print("pretrain Word2vec path is not given, loading")

            print("Word2Vec Training...")
            # using the CBOW model of word2vec, because we don't use to predict
            w2v_model = gensim.models.Word2Vec(sentences=corpus_clean_text, size=embedding_size, min_count=1)
            print("Word2Vec Train complete")
            self.w2v_model = w2v_model.wv

        self.compute_avgw2v_for_docs(preprocess_document_collection)

    def compute_avgw2v_for_docs(self, preprocess_document_collection):
        avg_w2v_model = gensim.models.keyedvectors.Word2VecKeyedVectors(vector_size=self.embedding_size)
        corpus_clean_text = preprocess_document_collection.get_all_preprocess_documents_text_words()

        avg_vector_list = []
        avg_index_str_list = []
        for index, doc_clean_text in enumerate(corpus_clean_text):
            avg_vector = self.get_avg_w2v_vec(doc_clean_text)
            avg_vector_list.append(avg_vector)
            avg_index_str_list.append(str(index))
            print("index=%d avg_vec" % index)
        print("avg_index_str_list len=%d avg_vector_list=%d" % (len(avg_index_str_list), len(avg_vector_list)))

        avg_w2v_model.add(entities=avg_index_str_list, weights=avg_vector_list, replace=True)
        self.avg_w2v_model = avg_w2v_model

    def get_avg_w2v_vec(self, word_str_list):
        """
        get the average word2vec for list of str
        :param word_str_list:  list of str
        :return: np.array()
        """
        # todo: add a empty zero vectors result.
        if len(word_str_list) == 0:
            return self.NP_VECTOR_NOT_EXIST

        valid_word_str_list = []

        for word in word_str_list:
            if word not in self.w2v_model.vocab:
                continue
            valid_word_str_list.append(word)

        if len(valid_word_str_list) == 0:
            return self.NP_VECTOR_NOT_EXIST

        doc_word_vectors = self.w2v_model[valid_word_str_list]

        avg_vector = np.mean(doc_word_vectors, axis=0)
        return avg_vector

    def train_from_doc_collection_with_preprocessor(self, doc_collection, **config):
        embedding_size = self.DEFAULT_EMBEDDING_SIZE
        pretrain_w2v_path = None
        tune_pretrain_w2v = True
        pretrain_binary = True
        if "embedding_size" in config.keys():
            embedding_size = config["embedding_size"]
        if "pretrain_w2v_path" in config.keys():
            pretrain_w2v_path = config["pretrain_w2v_path"]

        if "tune_pretrain_w2v" in config.keys():
            tune_pretrain_w2v = config["tune_pretrain_w2v"]

        if "pretrain_binary" in config.keys():
            pretrain_binary = config["pretrain_binary"]

        print("start training")

        if isinstance(doc_collection, PreprocessMultiFieldDocumentCollection):
            self.train_from_document_collection(preprocess_document_collection=doc_collection,
                                                embedding_size=embedding_size,
                                                pretrain_w2v_path=pretrain_w2v_path,
                                                tune_pretrain_w2v=tune_pretrain_w2v,
                                                pretrain_binary=pretrain_binary)

    def save(self, model_dir_path):
        """
        save model to the model_dir_path
        :param model_dir_path: the dir to save the model
        :return:
        """
        super().save(model_dir_path)

        self.__init_sub_model_path__()

        self.dict.save(self.dictionary_path)

        print("build dictionary in %s" % self.dictionary_path)

        gensim.corpora.MmCorpus.serialize(self.corpus_path, self.corpus)
        print("save the corpus to %s" % self.corpus_path)

        self.w2v_model.save(self.w2v_model_path)
        print("Word2vec Training finish , save to %s" % self.w2v_model_path)
        self.avg_w2v_model.save(self.avg_w2v_model_path)
        print("AVGWord2vec Training finish , save to %s" % self.avg_w2v_model_path)

        print("entity collection saving...")
        self.preprocess_doc_collection.save(self.entity_collection_path)
        print(
            "entity collection finish saving , save to %s, %r" % (
                self.entity_collection_path, self.preprocess_doc_collection))

    def support_text2vector(self):
        return True

    def string2vector(self, doc):
        query_words = self.preprocessor.clean(doc)
        avg_vector = self.get_avg_w2v_vec(query_words)
        return avg_vector

    def words2vector(self, words):
        avg_vector = self.get_avg_w2v_vec([w.lower() for w in words])
        return avg_vector



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
-----------------------------------------
@Author: XSS
@Email: 18212010042@fudan.edu.cn
@Created: 2020/06/28
------------------------------------------
@Modify: 2020/06/28
------------------------------------------
@Description:
"""
import re
import time
from collections import Counter
from pathlib import Path

import gensim
import numpy as np
from gensim import corpora
from nltk.corpus import stopwords
from nltk.stem import WordNetLemmatizer
from nltk.tokenize import sent_tokenize
from pattern.text import parsetree

"""
   给定一段文本，从中抽取实体,如果是其他格式的文本，请转成纯文本的输入
"""


class EntityExtractorForText:
    MAX_STRING_LEN = 60
    MAX_WORD_NUM = 8
    invalid_content = [
        "0x",
        "|",
        "^ ",
        "_ ",
        ":[ ",
        "\\u",
        "\\u",
        "<",
        ">",
        "</",
        "/>",
        "//",
        "::",
    ]
    VALID_CHAR_STRING = "qwertyuiopasdfghjklzxcvbnm./1234567890()@_<>/- "

    def __init__(self, tf_idf_model_dir=None, corpus_list=None):

        self.n_gram_num = 3
        self.tfidf_sim = 0.5
        self.stopWords = set(stopwords.words("english"))
        self.lemmatizer = WordNetLemmatizer()
        self.min_e = 1
        self.min_p = 2
        self.tf_idf_flag = False

        if tf_idf_model_dir:
            self.tf_idf_model_dir = tf_idf_model_dir
            self.dictionary_path = str(Path(self.tf_idf_model_dir) / "dictionary.bin")
            self.corpus_path = str(Path(self.tf_idf_model_dir) / " corpus.mm")
            self.model_path = str(Path(self.tf_idf_model_dir) / "ifidf.model")
            self.tf_idf_flag = True
            self.corpus_list = corpus_list
            self.__init_model__()

    def update_n_gram_num(self, n_gram_num):
        self.n_gram_num = n_gram_num

    def update_tfidf_sim(self, tfidf_sim):
        self.tfidf_sim = tfidf_sim

    def update_min_e(self, min_e):
        self.min_e = min_e

    def update_min_p(self, min_p):
        self.min_p = min_p

    def __init_model__(self):
        if Path(self.dictionary_path).exists():
            self.dictionary = corpora.Dictionary.load(self.dictionary_path)
        else:
            self.dictionary = self.generate_dictionary(self.corpus_list)
        if Path(self.corpus_path).exists():
            self.corpus = gensim.corpora.MmCorpus(self.corpus_path)

        if Path(self.model_path).exists():
            self.tfidf_model = gensim.models.TfidfModel.load(self.model_path)
        else:
            print(self.corpus)
            self.tfidf_model = gensim.models.TfidfModel(corpus=self.corpus, id2word=self.dictionary, smartirs="dtc")

    def extract_chunk(self, text):
        """
        return all the chunk extracted form the text
        :param text: the text
        :return: list of chunk of object pattern.text.Chunk
        """
        result = []
        pst = parsetree(text)
        for sentence in pst:
            for chunk in sentence.chunks:
                if chunk.type == "NP":
                    result.append(chunk)
                    # print chunk.type, [(w.string, w.type) for w in chunk.words]
        return result

    def filter_chunk(self, chunk_list):
        """
        return all the chunk extracted form the text
        :param text: the text
        :return: list of chunk of object pattern.text.Chunk
        """
        result = []

        for chunk in chunk_list:
            # print(chunk.words[0])
            # if chunk.words[0].string=="The":
            #     chunk.words.remove(chunk.words[0])
            if self.is_valid_chunk_string(chunk.string):
                result.append(chunk)
        return result

    def is_valid_chunk_string(self, chunk_string):
        if chunk_string == "" or chunk_string == None:
            return False
        chunk_string = chunk_string.lower()

        if chunk_string in self.stopWords:
            return False
        for char in chunk_string:
            if char not in self.VALID_CHAR_STRING:
                return False
        if len(chunk_string.split(" ")) >= self.MAX_WORD_NUM:
            return False
        if chunk_string.endswith(".") and len(chunk_string) == 2:
            return False
        if len(chunk_string) > self.MAX_STRING_LEN or len(chunk_string) <= 2:
            return False

        if "https://" in chunk_string or "http://" in chunk_string:
            return False

        if "(" in chunk_string and ")" not in chunk_string:
            return False
        if ")" in chunk_string and "(" not in chunk_string:
            return False
        for invalid in self.invalid_content:
            if invalid in chunk_string:
                return False
        for word in chunk_string.replace(".", " ").replace("x", " ").split(" "):
            if word.isdigit():
                return False
        return True

    # print(chunk.string+" not in stopwords")
    def lemmatize_chunk(self, chunk_string):
        words = chunk_string.split(" ")
        new_words = []
        for word in words:
            new_words.append(self.lemmatizer.lemmatize(word))

        return " ".join(new_words)

    def get_clean_entity_name_for_string(self, chunk_string):
        """
        get the valid entity name, for example, "an Activity"=>"Activity","activities"=>"activity"
        :param chunk_string:
        :return: None, the cleaned entity name can be got. otherwise, return the clean entity
        """

        if self.is_valid_chunk_string(chunk_string) == False:
            return None
        return self.clean_np_text(chunk_string)

    def extract_chunks_strings(self, text):
        if text == "" or text == None:
            return []
        chunk_list = self.extract_chunk(text)
        chunk_list = self.filter_chunk(chunk_list)
        chunk_strings = [chunk.string for chunk in chunk_list]
        return list(set(chunk_strings))

    def extract_clean_entity_name_from_text(self, text):
        chunk_list = self.extract_chunks_strings(text)
        entity_name_list = []
        for chunk in chunk_list:
            entity_name = self.get_clean_entity_name_for_string(chunk)
            if entity_name:
                entity_name_list.append(entity_name)
        return list(set(entity_name_list))

    def is_valid_date(self, input_str):
        """
        Determine if it is a valid date string
        """
        word_list = input_str.split(' ', 1)
        for w in word_list:
            try:
                if ":" in w:
                    time.strptime(w, "%Y-%m-%d %H:%M:%S")
                else:
                    time.strptime(w, "%Y-%m-%d")
                return True
            except:
                pass
        return False

    def is_link_by_num(self, np_text):
        pattern = re.compile(r'(\d+)-(\d+)')
        m = pattern.findall(np_text)
        if m is None or len(m) == 0:
            return False

        return True

    def remove_start_or_end_stop_word_for_one_time(self, np_text):
        word_list = np_text.lower().split(' ')
        if word_list[0] in self.stopWords:
            word_list = word_list[1:]
        if len(word_list) == 0:
            return None
        if word_list[-1] in self.stopWords:
            word_list = word_list[:-1]
        if len(word_list) == 0:
            return None

        return " ".join(word_list)

    def remove_start_dot(self, np_text):
        while len(np_text) > 0 and np_text[0] == '.':
            np_text = np_text[1:]
        return np_text

    def remove_other_start(self, np_text):
        if np_text.find('0x') == 0 or np_text.find('/ ') == 0 or np_text.find('1/') == 0:
            return True
        return False

    def end_with_point(self, np_text):
        if np_text[-1] == '.':
            return np_text[0:-1]
        return np_text

    def remove_all_stop_words_in_start_and_end(self, np_text):
        while True:
            new_np_text = self.remove_start_or_end_stop_word_for_one_time(np_text)
            if new_np_text == None:
                return None

            if np_text == new_np_text:
                return np_text
            np_text = new_np_text

    def clean_np_text(self, np_text):

        np_text = np_text.lower()
        '''
        eg: merge
        3166-1 two-letter country code
        3166-1 two letters country code
        to lower case
        start with stop word
        remove ‘-’
        lemmatize_chunk
        '''
        np_text = self.remove_start_dot(np_text)
        np_text = self.end_with_point(np_text)
        if np_text.isspace():
            return None
        if self.remove_other_start(np_text):
            return None
        is_valid_date = self.is_valid_date(np_text)
        np_text = self.lemmatize_chunk(np_text)
        is_link_by_num = self.is_link_by_num(np_text)
        if is_valid_date or is_link_by_num:
            return None
        np_text = self.remove_all_stop_words_in_start_and_end(np_text)
        if np_text == None:
            return None
        words = np_text.split(" ")
        if np_text[0] == "@" and len(words) >= 2:
            np_text = words[0] + " ".join(words[1:])
        if np_text == "extended data":
            np_text = "data"
        np_text = np_text.replace("-", " ")
        return np_text

    def extract_single_verb(self, text):
        """
        return all the chunk extracted form the text
        :param text: the text
        :return: list of chunk of object pattern.text.Chunk
        """
        result = []
        pst = parsetree(text)
        for sentence in pst:
            for chunk in sentence.chunks:
                if chunk.type == "VP":
                    result.append(chunk.string.split(" ")[0])

        candidates = set(result)

        final_result = []
        for candidate in candidates:
            candidate = self.lemmatizer.lemmatize(candidate)
            if candidate not in self.stopWords:
                final_result.append(candidate)

        return list(set(final_result))

    def extract_verb_and_entity_name_from_text(self, text):

        verb_result = []
        np_result = []

        pst = parsetree(text)
        for sentence in pst:
            for chunk in sentence.chunks:
                if chunk.type == "NP":
                    np_result.append(chunk)
                    continue
                if chunk.type == "VP":
                    word_tagged_list = chunk.tagged
                    for word in word_tagged_list:
                        if word[1][0] == "V":
                            verb_result.append(word[0])
                    continue

        verb_result = set(verb_result)

        final_result = []
        for candidate in verb_result:
            candidate = self.lemmatizer.lemmatize(candidate)
            if candidate not in self.stopWords:
                final_result.append(candidate)

        for np in np_result:
            entity_name = self.get_clean_entity_name_for_string(np.string)
            if entity_name:
                final_result.append(entity_name)

        return list(set(final_result))

    def expand_the_chunk_by_words(self, final_chunk_list):
        final_set = []
        for chunk in final_chunk_list:
            final_set.append(chunk)
            for word in chunk.split(" "):
                if word not in self.stopWords:
                    final_set.append(word)
        print("word set", final_set)
        return list(set(final_set))

    def get_all_possible_key_word_from_text(self, text):
        key_words_list = self.extract_verb_and_entity_name_from_text(text)
        whole_words = " ".join(key_words_list)

        final_key_words_list = self.expand_the_chunk_by_words(key_words_list)
        final_key_words_list.append(whole_words)
        return final_key_words_list

    def clean_text(self, text):
        text = re.sub('\[.*?\]', '', text)

        punctuation = ['【', '】', ')', '(', '、', '，', '“', '”', '。', '\n', '《', '》', ' ', '-', '！', '？', '.', '\'', '[',
                       ']', ':', "：", '/', '.', '"', '\u3000', '’', '．', ',', '…', '?', '+', "(", ")", "=", "："]
        for i in punctuation:
            text = text.replace(i, " ")
        text = " ".join(text.split())
        return text

    # n-gram-list
    def create_ngram_list(self, sentence, n):
        """
        creat ngram list
        :param sentence:  clean sentence, str
        :param n: n of ngram
        :return:
        """
        ngram_list = []
        for ngram_num in range(1, n + 1):
            if len(sentence) <= ngram_num:
                ngram_list.append(sentence)
            else:
                input_words = sentence.split()
                for tmp in zip(*[input_words[i:] for i in range(ngram_num)]):
                    tmp = " ".join(tmp)
                    if tmp and tmp.lower() not in self.stopWords:
                        ngram_list.append(tmp)
        return ngram_list

    def find_new_words(self, ngram_list: list, clean_sentence: str):
        """
        find new words by PMI and Entropy_left_right
        :param ngram_list: ngram_list,list
        :param clean_sentence: sentences, str
        :return:
        """
        if not ngram_list:
            return []
        min_e = self.min_e
        min_p = self.min_p
        words_freq = dict(Counter(ngram_list))
        candidate = self.PMI_filter(words_freq, min_p)
        entity_list = self.Entropy_left_right_filter(candidate, clean_sentence, min_e)
        return entity_list

    def PMI_filter(self, word_freq_dic, min_p):
        """
        To get words witch  PMI  over the threshold
        input: word frequency dict , min threshold of PMI
        output: condinated word list

        """
        new_words = []
        for words in word_freq_dic:
            # print(words)
            split_word = words.split()
            if len(split_word) == 1:
                pass
            else:
                p_x_y = min(
                    [word_freq_dic.get(" ".join(split_word[:i]), 0) * word_freq_dic.get(" ".join(split_word[i:]), 0) for
                     i in range(1, len(split_word))])
                # print("p_x_y:", p_x_y)
                mpi = p_x_y / word_freq_dic.get(words)
                if mpi > min_p:
                    new_words.append(words)
        return new_words

    def calculate_entropy(self, char_list):
        """
        To calculate entropy for  list  of char
        input: char list
        output: entropy of the list  of char
        """
        char_freq_dic = dict(Counter(char_list))
        entropy = (-1) * sum(
            [char_freq_dic.get(i) / len(char_list) * np.log2(char_freq_dic.get(i) / len(char_list)) for i in
             char_freq_dic])
        return entropy

    def Entropy_left_right_filter(self, condinate_words, text, min_entropy):
        """
        To filter the final new words from the condinated word list by entropy threshold
        input:  condinated word list ,min threshold of Entropy of left or right
        output: final word list
        """
        final_words = []
        # print(text)
        for word in condinate_words:
            try:

                left_right_char = re.findall('([^\s]*)\s%s\s([^\s]*)' % word, text)
                left_char = [i[0] for i in left_right_char]
                left_entropy = self.calculate_entropy(left_char)

                right_char = [i[1] for i in left_right_char]
                right_entropy = self.calculate_entropy(right_char)

                if min(right_entropy, left_entropy) > min_entropy:
                    final_words.append(word)
            except:
                pass
        return final_words

    def filter_by_tf_idf(self, ngram_list):
        sentence_bow, missing_words = self.dictionary.doc2bow(ngram_list, return_missing=True,
                                                              allow_update=True)
        sentence_tfidf_vec = list(self.tfidf_model[sentence_bow])
        sentence_tfidf_vec = [sim for index, sim in sentence_tfidf_vec]
        np_array = np.array(sentence_tfidf_vec)
        over_thred = np.where(np_array > self.tfidf_sim)
        sentence_bow_filter = np.array(sentence_bow)[over_thred]
        entity = []
        for key, sim in list(sentence_bow_filter):
            if key in self.dictionary:
                entity.append(self.dictionary[key])
        return entity

    def generate_dictionary(self, text_list: list):
        """
        generate n-gram dictionary
        :param text_list: crops
        :return: dictionary
        """
        crops = []
        for text in text_list:
            ngram_list = []
            for sentence in sent_tokenize(text.replace('\n', '')):
                clean_sentence = self.clean_text(sentence)
                if not clean_sentence:
                    continue
                tmp_ngram_list = self.create_ngram_list(clean_sentence, 3)
                if tmp_ngram_list:
                    print(sentence)
                    ngram_list += tmp_ngram_list
            if ngram_list:
                crops.append(ngram_list)
        dictionary = corpora.Dictionary(crops)
        dictionary.save(self.dictionary_path)
        self.corpus = [dictionary.doc2bow(text) for text in crops]
        gensim.corpora.MmCorpus.serialize(self.corpus_path, self.corpus)
        return dictionary

    def entity_add(self, new_entity_list, entity_dic, extract_way="chunk"):
        for entity in new_entity_list:
            if entity not in entity_dic.keys():
                entity_dic[entity] = {extract_way}
            else:
                entity_dic[entity].add(extract_way)
        return entity_dic

    def extract(self, text):
        # self.extractor = EntityExtractor()
        all_clean_sentence = []
        all_n_gram = []
        entity_dic = {}
        for sentence in sent_tokenize(text.replace('\n', '')):
            clean_sentence = self.clean_text(text)
            if clean_sentence:
                chunk_strings = self.extract_clean_entity_name_from_text(sentence)
                # chunk_strings, _ = self.extractor.extract_from_sentence(sentence)
                entity_dic = self.entity_add(chunk_strings, entity_dic, "chunk")
                all_clean_sentence.append(clean_sentence)
                ngram_list = self.create_ngram_list(clean_sentence, self.n_gram_num)
                if not ngram_list:
                    continue
                all_n_gram += ngram_list
                if not self.tf_idf_flag:
                    continue
                entity_tf_idf = self.filter_by_tf_idf(ngram_list)
                entity_dic = self.entity_add(entity_tf_idf, entity_dic, "tf_idf")

        entity_new = self.find_new_words(all_n_gram, " ".join(all_clean_sentence))
        entity_dic = self.entity_add(entity_new, entity_dic, "new_words")

        return entity_dic



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
-----------------------------------------
@Author: XSS
@Email: 18212010042@fudan.edu.cn
@Created: 2020/06/28
------------------------------------------
@Modify: 2020/06/28
------------------------------------------
@Description:
"""
from unittest import TestCase

from sekg.text.extractor.domain_entity.entity_extractor_for_text import EntityExtractorForText


class TestEntityExtractorForText(TestCase):
    def test_extract(self):
        sent = """Specifically, workspaces are designed for cyclical workloads - such as training neural networks - as
            they allow for off-heap memory reuse (instead of continually allocating and deallocating memory on each iteration
            of the loop). 
"""

        extractor = EntityExtractorForText()
        terms = extractor.extract(sent)
        print(terms)
